function [ipx, ipw] = ip_triangle(nip)
%IP_TRIANGLE Integration rules (points & weights) for triangular elements


%
% ipx(nip, 2): coordinates of integration points
% ipw(nip):    Gaussian quadrature weights
%

%   Part of MILAMIN: MATLAB-based FEM solver for large problems, Version 1.0
%   Copyright (C) 2007, M. Dabrowski, M. Krotkiewski, D.W. Schmid
%   University of Oslo, Physics of Geological Processes
%   http://milamin.org
%   See License file for terms of use.

% see e.g.: 
% Dunavant, D. A. 1985. High-Degree efficient symmetrical Gaussian quadrature rules for the triangle. Int. J. Numer. Methods Eng. 21, 6, 1129--1148.

switch nip;
    case 1;
        ipx(1,1) = 1/3;
        ipx(1,2) = 1/3;

        ipw(1)   = 0.5;

    case 3;
        ipx(1,1) = 1/6;
        ipx(1,2) = 1/6;
        ipx(2,1) = 2/3;
        ipx(2,2) = 1/6;
        ipx(3,1) = 1/6;
        ipx(3,2) = 2/3;

        ipw(1) = 1/6;
        ipw(2) = 1/6;
        ipw(3) = 1/6;

    case 4;
        ipx(1,1:2) = [1.0d0/3.0d0, 1.0d0/3.0d0];
        ipx(2,1:2) = [0.6d0,             0.2d0];
        ipx(3,1:2) = [0.2d0,             0.6d0];
        ipx(4,1:2) = [0.2d0,             0.2d0];

        ipw(1)  = -27.0d0/48.0d0;
        ipw(2)  =  25.0d0/48.0d0;
        ipw(3)  = ipw(2);
        ipw(4)  = ipw(3);
        ipw     = 0.5d0*ipw;

    case 6;
        g1 = (8-sqrt(10) + sqrt(38-44*sqrt(2/5)))/18;
        g2 = (8-sqrt(10) - sqrt(38-44*sqrt(2/5)))/18;
        ipx(1,1) = 1-2*g1;                  %0.108103018168070;
        ipx(1,2) = g1;                      %0.445948490915965;
        ipx(2,1) = g1;                      %0.445948490915965;
        ipx(2,2) = 1-2*g1;                  %0.108103018168070;
        ipx(3,1) = g1;                      %0.445948490915965;
        ipx(3,2) = g1;                      %0.445948490915965;
        ipx(4,1) = 1-2*g2;                  %0.816847572980459;
        ipx(4,2) = g2;                      %0.091576213509771;
        ipx(5,1) = g2;                      %0.091576213509771;
        ipx(5,2) = 1-2*g2;                  %0.816847572980459;
        ipx(6,1) = g2;                      %0.091576213509771;
        ipx(6,2) = g2;                      %0.091576213509771;

        w1 = (620+sqrt(213125-53320*sqrt(10)))/3720;
        w2 = (620-sqrt(213125-53320*sqrt(10)))/3720;
        ipw(1) =  w1;                       %0.223381589678011;
        ipw(2) =  w1;                       %0.223381589678011;
        ipw(3) =  w1;                       %0.223381589678011;
        ipw(4) =  w2;                       %0.109951743655322;
        ipw(5) =  w2;                       %0.109951743655322;
        ipw(6) =  w2;                       %0.109951743655322;
        ipw     = 0.5d0*ipw;

    case 7;
        g1 = (6-sqrt(15))/21;
        g2 = (6+sqrt(15))/21;
        ipx(1,1) = 1/3;                     %0.333333333333333;
        ipx(1,2) = 1/3;                     %0.333333333333333;
        ipx(2,1) = 1-2*g1;                  %0.797426985353087;
        ipx(2,2) = g1;                      %0.101286507323456;
        ipx(3,1) = g1;                      %0.101286507323456;
        ipx(3,2) = 1-2*g1;                  %0.797426985353087;
        ipx(4,1) = g1;                      %0.101286507323456;
        ipx(4,2) = g1;                      %0.101286507323456;
        ipx(5,1) = 1-2*g2;                  %0.059715871789770;
        ipx(5,2) = g2;                      %0.470142064105115;
        ipx(6,1) = g2;                      %0.470142064105115;
        ipx(6,2) = 1-2*g2;                  %0.059715871789770;
        ipx(7,1) = g2;                      %0.470142064105115;
        ipx(7,2) = g2;                      %0.470142064105115;

        w1 = (155-sqrt(15))/1200;
        w2 = (155+sqrt(15))/1200;
        ipw(1) = 0.225;
        ipw(2) = w1;                        %0.125939180544827;
        ipw(3) = w1;                        %0.125939180544827;
        ipw(4) = w1;                        %0.125939180544827;
        ipw(5) = w2;                        %0.132394152788506;
        ipw(6) = w2;                        %0.132394152788506;
        ipw(7) = w2;                        %0.132394152788506;
        ipw = 0.5*ipw;

    case 77 %LABATTO
        ipx(1,1) = 0.d0;
        ipx(1,2) = 0.d0;
        ipx(2,1) = 1.d0;
        ipx(2,2) = 0.d0;
        ipx(3,1) = 0.d0;
        ipx(3,2) = 1.d0;
        ipx(4,1) = 1.d0/2.d0;
        ipx(4,2) = 1.d0/2.d0;
        ipx(5,1) = 1.d0/2.d0;
        ipx(5,2) = 0.d0;
        ipx(6,1) = 0.d0;
        ipx(6,2) = 1.d0/2.d0;
        ipx(7,1) = 1.d0/3.d0;
        ipx(7,2) = 1.d0/3.d0;

        ipw(1)  = 1.d0/40.d0;
        ipw(2)  = 1.d0/40.d0;
        ipw(3)  = 1.d0/40.d0;
        ipw(4)  = 1.d0/15.d0;
        ipw(5)  = 1.d0/15.d0;
        ipw(6)  = 1.d0/15.d0;
        ipw(7)  = 9.d0/40.d0;

    case 12
        ipx(1,1) = 0.873821971016996;
        ipx(1,2) = 0.063089014491502;
        ipx(2,1) = 0.063089014491502;
        ipx(2,2) = 0.873821971016996;
        ipx(3,1) = 0.063089014491502;
        ipx(3,2) = 0.063089014491502;
        ipx(4,1) = 0.501426509658179;
        ipx(4,2) = 0.249286745170910;
        ipx(5,1) = 0.249286745170910;
        ipx(5,2) = 0.501426509658179;
        ipx(6,1) = 0.249286745170910;
        ipx(6,2) = 0.249286745170910;
        ipx(7,1) = 0.636502499121399;
        ipx(7,2) = 0.310352451033785;
        ipx(8,1) = 0.310352451033785;
        ipx(8,2) = 0.636502499121399;
        ipx(9,1) = 0.636502499121399;
        ipx(9,2) = 0.053145049844816;
        ipx(10,1) = 0.053145049844816;
        ipx(10,2) = 0.636502499121399;
        ipx(11,1) = 0.310352451033785;
        ipx(11,2) = 0.053145049844816;
        ipx(12,1) = 0.053145049844816;
        ipx(12,2) = 0.310352451033785;

        ipw(1)   = 0.050844906370207;
        ipw(2)   = 0.050844906370207;
        ipw(3)   = 0.050844906370207;
        ipw(4)   = 0.116786275726379;
        ipw(5)   = 0.116786275726379;
        ipw(6)   = 0.116786275726379;
        ipw(7)    = 0.082851075618374;
        ipw(8)    = 0.082851075618374;
        ipw(9)    = 0.082851075618374;
        ipw(10)   = 0.082851075618374;
        ipw(11)   = 0.082851075618374;
        ipw(12)   = 0.082851075618374;

        ipw = 0.5*ipw;

    case 25
        ipx(1,1) = 0.333333333333333;
        ipx(1,2) = 0.333333333333333;
        ipx(2,1) = 0.028844733232685;
        ipx(2,2) = 0.485577633383657;
        ipx(3,1) = 0.485577633383657;
        ipx(3,2) = 0.028844733232685;
        ipx(4,1) = 0.485577633383657;
        ipx(4,2) = 0.485577633383657;
        ipx(5,1) = 0.781036849029926;
        ipx(5,2) = 0.109481575485037;
        ipx(6,1) = 0.109481575485037;
        ipx(6,2) = 0.781036849029926;
        ipx(7,1) = 0.109481575485037;
        ipx(7,2) = 0.109481575485037;
        ipx(8,1) = 0.141707219414880;
        ipx(8,2) = 0.307939838764121;
        ipx(9,2) = 0.141707219414880;
        ipx(9,1) = 0.307939838764121;
        ipx(10,1) = 0.141707219414880;
        ipx(10,2) = 0.550352941820999;
        ipx(11,2) = 0.141707219414880;
        ipx(11,1) = 0.550352941820999;
        ipx(12,1) = 0.307939838764121;
        ipx(12,2) = 0.550352941820999;
        ipx(13,2) = 0.307939838764121;
        ipx(13,1) = 0.550352941820999;
        ipx(14,1) = 0.025003534762686;
        ipx(14,2) = 0.246672560639903;
        ipx(15,2) = 0.025003534762686;
        ipx(15,1) = 0.246672560639903;
        ipx(16,1) = 0.025003534762686;
        ipx(16,2) = 0.728323904597411;
        ipx(17,2) = 0.025003534762686;
        ipx(17,1) = 0.728323904597411;
        ipx(18,1) = 0.246672560639903;
        ipx(18,2) = 0.728323904597411;
        ipx(19,2) = 0.246672560639903;
        ipx(19,1) = 0.728323904597411;
        ipx(20,1) = 0.009540815400299;
        ipx(20,2) = 0.066803251012200;
        ipx(21,2) = 0.009540815400299;
        ipx(21,1) = 0.066803251012200;
        ipx(22,1) = 0.009540815400299;
        ipx(22,2) = 0.923655933587500;
        ipx(23,2) = 0.009540815400299;
        ipx(23,1) = 0.923655933587500;
        ipx(24,1) = 0.066803251012200;
        ipx(24,2) = 0.923655933587500;
        ipx(25,2) = 0.066803251012200;
        ipx(25,1) = 0.923655933587500;

        ipw(1)   = 0.090817990382754;
        ipw(2)   = 0.036725957756467;
        ipw(3)   = 0.036725957756467;
        ipw(4)   = 0.036725957756467;
        ipw(5)   = 0.045321059435528;
        ipw(6)   = 0.045321059435528;
        ipw(7)   = 0.045321059435528;
        ipw(8)   = 0.072757916845420;
        ipw(9)   = 0.072757916845420;
        ipw(10)  = 0.072757916845420;
        ipw(11)  = 0.072757916845420;
        ipw(12)  = 0.072757916845420;
        ipw(13)  = 0.072757916845420;
        ipw(14)   = 0.028327242531057;
        ipw(15)   = 0.028327242531057;
        ipw(16)   = 0.028327242531057;
        ipw(17)   = 0.028327242531057;
        ipw(18)   = 0.028327242531057;
        ipw(19)   = 0.028327242531057;
        ipw(20)  = 0.009421666963733;
        ipw(21)  = 0.009421666963733;
        ipw(22)  = 0.009421666963733;
        ipw(23)  = 0.009421666963733;
        ipw(24)  = 0.009421666963733;
        ipw(25)  = 0.009421666963733;
        ipw = 0.5*ipw;
        
    otherwise
        error('Unknown integration rule')
        
end

end